@@ -1,40 +1,56 @@
-package Vroom;
+##
+# name:      Vroom
+# abstract:  Slide Shows in Vim
+# author:    Ingy döt Net <ingy@ingy.net>
+# license:   perl
+# copyright: 2008, 2009, 2010, 2011
+
 use 5.006001;
-use Vroom::OO -base;
+package Vroom;
+use Vroom::Mo;
 
-our $VERSION = '0.25';
+our $VERSION = '0.26';
+
+use File::HomeDir 0.97;
+use IO::All 0.44;
+use Template::Toolkit::Simple 0.13;
+use Term::Size 0.2;
+use YAML::XS 0.37;
 
-use IO::All;
-use YAML::XS;
 use Getopt::Long;
-use File::HomeDir;
 use Cwd;
 use Carp;
 
-has input => 'slides.vroom';
-has stream => '';
-has ext => '';
-has help => 0;
-has clean => 0;
-has compile => 0;
-has sample => 0;
-has run => 0;
-has html => 0;
-has text => 0;
-has ghpublish => 0;
-has start => 0;
-has digits => 0;
-has skip => 0;
-has config => {
-    title => 'Untitled Presentation',
-    height => 24,
-    width => 80,
-    list_indent => 10,
-    skip => 0,
-    vim => 'vim',
-    vimrc => '',
-    gvimrc => '',
-};
+has input => (default => sub {'slides.vroom'});
+has stream => (default => sub {''});
+has ext => (default => sub {''});
+has help => (default => sub{0});
+has clean => (default => sub{0});
+has compile => (default => sub{0});
+has sample => (default => sub{0});
+has run => (default => sub{0});
+has html => (default => sub{0});
+has text => (default => sub{0});
+has ghpublish => (default => sub{0});
+has start => (default => sub{0});
+has digits => (default => sub{0});
+has skip => (default => sub{0});
+has config => (
+    default => sub {
+        +{
+            title => 'Untitled Presentation',
+            height => 24,
+            width => 80,
+            list_indent => 10,
+            skip => 0,
+            vim => 'vim',
+            vimrc => '',
+            gvimrc => '',
+            script => '',
+            auto_size => 0,
+        };
+    }
+);
 
 sub usage {
     return <<'...';
@@ -127,6 +143,8 @@ sub cleanUp {
     unlink('.vimrc');
     unlink('.gvimrc');
     unlink('run.slide');
+    io->dir('bin')->rmtree;
+    io->dir('done')->rmtree;
 }
 
 sub cleanAll {
@@ -185,6 +203,7 @@ sub makeSlides {
     $self->getInput;
     $self->buildSlides;
     $self->writeVimrc;
+    $self->writeScriptRunner;
     $self->writeHelp;
 }
 
@@ -209,7 +228,6 @@ sub makeText {
 
 sub makeHTML {
     my $self = shift;
-    require Template::Toolkit::Simple;
     $self->cleanAll;
     $self->makeSlides;
     io('html')->mkdir;
@@ -385,13 +403,19 @@ sub buildSlides {
         $raw_slide = $self->padVertical($raw_slide);
 
         my @slides;
+        my @scripts;
         my $slide = '';
-        for (split /^\+/m, $raw_slide) {
+        for my $part (split /^\+/m, $raw_slide) {
             $slide = '' if $config->{replace};
-            $slide .= $_;
+            my $script = '';
+            if ($self->config->{script}) {
+                ($part, $script) = $self->parseScript($part);
+            }
+            $slide .= $part;
             $slide = $self->padVertical($slide)
                 if $config->{replace};
             push @slides, $slide;
+            push @scripts, $script;
         }
 
         my $base_name = $self->formatNumber($number);
@@ -417,11 +441,28 @@ sub buildSlides {
                 : $i == @slides
                     ? 'z'
                     : $suf;
-            io("$base_name$suf" . $self->ext)->print($slide);
+            my $file_name = "$base_name$suf" . $self->ext;
+            io($file_name)->print($slide);
+            if (my $script = shift @scripts) {
+                io("bin/$file_name")->assert->print($script);
+            }
         }
     }
 }
 
+sub parseScript {
+    my $self = shift;
+    my $text = shift;
+    chomp $text;
+    $text .= "\n";
+    my $script = '';
+    my $delim = $self->config->{script};
+    while ($text =~ s/^[\ \t]*\Q$delim\E(.*\n)//m) {
+        $script .= $1;
+    }
+    return ($text, $script);
+}
+
 sub formatNumber {
     my $self = shift;
     my $number = shift;
@@ -480,10 +521,19 @@ sub applyOptions {
             %{$self->config},
             %{(YAML::XS::Load($slide))},
         };
+
+        if ($config->{auto_size}) {
+            my ($columns, $rows) = Term::Size::chars *STDOUT{IO};
+
+            $config->{width}  = $columns;
+            $config->{height} = $rows;
+        }
+
         $self->config($config);
         return '';
     }
 
+    $slide ||= '';
     if ($config->{center}) {
         $slide =~ s{^(\+?)\ *(.*?)\ *$}
                    {$1 . ' ' x (($self->config->{width} - length($2)) / 2) . $2}gem;
@@ -543,6 +593,30 @@ sub writeVimrc {
     my $home_vimrc = File::HomeDir->my_home . "/.vroom/vimrc";
     my $home_vimrc_content = -e $home_vimrc ? io($home_vimrc)->all : ''; 
 
+    my $next_cmd = $self->config->{script}
+        ? ':n<CR>:<CR>:call RunIf()<CR>:<CR>gg'
+        : ':n<CR>:<CR>gg';
+    my $script_functions = $self->config->{script} ? <<'...' : '';
+function RunIf()
+    let script = "bin/" . expand("%")
+    let done = "done/" . expand("%")
+    if filereadable(done)
+        return
+    endif
+    if filereadable(script)
+        call system("sh " . script)
+        call system("touch " . done)
+    endif
+    return
+endfunction
+
+function RunNow()
+    let done = "done/" . expand("%")
+    call system("rm -f " . done)
+    call RunIf()
+endfunction
+...
+
     die <<'...'
 The .vimrc in your current directory does not look like vroom created it.
 
@@ -556,10 +630,12 @@ time, and rerun vroom. You should not get this message again.
     $title =~ s/\s/\\ /g;
     io(".vimrc")->print(<<"...");
 " This .vimrc file was created by Vroom-$VERSION
-map <SPACE> :n<CR>:<CR>gg
+$script_functions
+map <SPACE> $next_cmd
 map <BACKSPACE> :N<CR>:<CR>gg
 map R :!vroom -run %<CR>
 map RR :!vroom -run %<CR>
+map AA :call RunNow()<CR>:<CR>
 map VV :!vroom -vroom<CR>
 map QQ :q!<CR>
 map OO :!open <cWORD><CR><CR>
@@ -590,6 +666,12 @@ $home_gvimrc_content
     }
 }
 
+sub writeScriptRunner {
+    my $self = shift;
+    return unless $self->config->{script};
+    mkdir 'done';
+}
+
 sub writeHelp {
     my $self = shift;
     io('.help')->print(<<'...');
@@ -640,6 +722,7 @@ title: Vroom!
 indent: 5
 height: 18
 width: 69
+#auto_size: 1
 skip: 0
 
 # The following options are for Gvim usage.
@@ -771,12 +854,6 @@ If it makes sense to you, run it. (at your own risk :)
 ...
 }
 
-=encoding utf8
-
-=head1 NAME
-
-Vroom - Slide Shows in Vim
-
 =head1 SYNOPSIS
 
     > mkdir MySlides    # Make a Directory for Your Slides
@@ -885,6 +962,7 @@ Here is an example slides.vroom file:
     height: 84
     width: 20
     # skip: 12      # Skip 12 slides. Useful when making slides.
+    # auto_size: 1  # Determines height/width automatically
     ---- center
     My Presentation
 
@@ -990,6 +1068,11 @@ show. Used for centering the content.
 The number of columns in the terminal you plan to use when presenting
 the show. Used for centering the content.
 
+=item auto_size: <0|1>
+
+When set to 1, the height/width options above will be ignored and
+determined each time you start the slideshow.
+
 =item indent: <number>
 
 All slides will be indented by this number of spaces by default.
@@ -1136,18 +1219,3 @@ ones above, to publish your slides to a gh-pages branch.
 
 You can see an example of a talk published to HTML and posted via gh-pages
 at L<http://ingydotnet.github.com/vroom-pm/>.
-
-=head1 AUTHOR
-
-Ingy döt Net <ingy@cpan.org>
-
-=head1 COPYRIGHT
-
-Copyright (c) 2008, 2009. Ingy döt Net.
-
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
-
-See http://www.perl.com/perl/misc/Artistic.html
-
-=cut
